@@ -16,12 +16,12 @@ remain, otherwise it is changed to zero.This yields the actual
 returns assuming instantaneous re-balancing at 12: 00:00 UTC
 daily.
 """
-
 import time
 
 import matplotlib.pyplot as plt
-import numpy
+import numpy as np
 import pandas as pd
+import plotly.graph_objects as go
 
 from python_backtesting_engine.Backtesting import StratFile
 
@@ -31,7 +31,8 @@ class Backtester:
     def __init__(self, data_file):
         self._file_loc = data_file[0]
         self._csv_df = pd.read_csv(self._file_loc, index_col=0, infer_datetime_format=True, parse_dates=True)
-        self._logical_df = StratFile.strat_file(self._csv_df.astype(numpy.number).pct_change(1))
+        self._logical_df = StratFile.strat_file(self._csv_df.astype(np.number).pct_change(1))
+        self._ret = self._csv_df.astype(np.number).pct_change(1)
 
     # %%
     @staticmethod
@@ -47,7 +48,7 @@ class Backtester:
         logic_df_ = logic_df.fillna(0)[1:]
 
         ret_ = ret.fillna(0)[:-1]
-        ret_ = ret_ + 1 # change from +10% == 0.10 to +10% == 1.10
+        ret_ = ret_ + 1  # change from +10% == 0.10 to +10% == 1.10
 
         # Element wise multiplication of matrices
         mult = logic_df_.reset_index(drop=True) * ret_.reset_index(drop=True)
@@ -55,26 +56,6 @@ class Backtester:
 
         return mult
 
-    # %%
-    def calc_stats(self):
-        """
-        :param  self:
-        :returns: [cum_df, algo_ret]: [dataframe of cumulative values, returns based off StratFile]
-        """
-        tic = time.process_time()
-        ret = self._csv_df.astype(numpy.number).pct_change(1)
-        algo_ret = self.calc_ret(self._logical_df, ret)
-        toc = time.process_time()
-        print(f'{(toc - tic) * 1000}ms')
-
-        cum_dict = self.val_each_asset(algo_ret)
-
-        cum_df = self.ending_vals(cum_dict)
-
-        sum_vals = self.daily_portfolio(cum_df)
-
-        self.plotter(cum_dict, cum_df, sum_vals)
-
     # %%
     @staticmethod
     def val_each_asset(algo_ret):
@@ -118,14 +99,44 @@ class Backtester:
 
         return sum_vals
 
+    #%%
+    @staticmethod
+    def sharpe(portfolio_val):
+        """
+        Computes Sharpe Ratio from dataframe of daily total portfolio value
+        :param portfolio_val: dataframe of daily portfolio values
+        :return: Sharpe Ratio
+        """
+        daily_ret = portfolio_val.astype(np.number).pct_change(1)
+        daily_ret.dropna(inplace=True)
+        sharpe_ratio = np.sqrt(365)*((np.mean(daily_ret)) / (np.std(daily_ret)))  # 365 --> 225 for stocks
+
+        return round(sharpe_ratio, 4)
+
     # %%
     @staticmethod
-    def plotter(data1, data2, data3):
-        """
+    def more_stats(cumulative_df, daily_tot):
+        end_vals = ((cumulative_df.iloc[-1] - 1)/1) * 100
+        max_ret = np.max(end_vals)
+        min_ret = np.min(end_vals)
+        avg_ret = np.mean(end_vals)
+        end_return = ((daily_tot.iloc[-1]-daily_tot.iloc[0])/daily_tot.iloc[0]) * 100
+        np.round(end_return, 4)
 
+        print(f"Max single asset return: {max_ret}% \n"
+              f"Min single asset loss: {min_ret}% \n"
+              f"Avg overall return: {avg_ret}% \n"
+              f"Overall Return: {end_return}% \n")
+
+    # %%
+
+    @staticmethod
+    def plotter(data1, data2, data3, data4):
+        """
         :param data1: cum_dict, data from val_each_asset
         :param data2: cum_df, data from ending_vals
         :param data3: sum_vals, data from daily_portfolio
+        :param data4: Sharpe Ratio
         :return: None
         """
 
@@ -152,6 +163,30 @@ class Backtester:
         ax3.set_ylabel(f'Portfolio Value (Dollars) [starting value of {len(data2.columns)}]')
         plt.show()
 
+        try:
+            if data4 < 0:
+                print("Sharpe Ratio is negative, strategy is not profitable \n"
+                      f"Sharpe Ratio = {data4}, ")
+            elif data4 >= 0:
+                # Show Sharpe Ratio
+                fig = go.Figure(go.Indicator(
+                    mode="gauge+number",
+                    value=data4,
+                    domain={'x': [0, 1], 'y': [0, 1]},
+                    gauge={'axis': {'range': [None, 4]},
+                           'steps': [
+                               {'range': [-1, 1], 'color': "red"},
+                               {'range': [1, 2], 'color': "yellow"},
+                               {'range': [2, 3], 'color': "green"},
+                               {'range': [3, 4], 'color': "green"}]},
+                    title={'text': "Sharpe Ratio"}))
+                fig.show()
+            elif data4 > 4:
+                print(f"Your sharpe ratio is so good it broke the dial gauge!!!! \n"
+                      f"Sharpe Ratio = {data4} ")
+        except TypeError:
+            print("Error Sharpe Ratio is an invalid number or not a number")
+
     #%%
     def calc_stats(self):
         """
@@ -159,7 +194,7 @@ class Backtester:
         :returns: [cum_df, algo_ret]: [dataframe of cumulative values, returns based off StratFile]
         """
         tic = time.process_time()
-        ret = self._csv_df.astype(numpy.number).pct_change(1)
+        ret = self._ret
         algo_ret = self.calc_ret(self._logical_df, ret)
         toc = time.process_time()
         print(f'{(toc - tic) * 1000}ms')
@@ -170,8 +205,8 @@ class Backtester:
 
         sum_vals = self.daily_portfolio(cum_df)
 
-        self.plotter(cum_dict, cum_df, sum_vals)
-
-
+        sharpe_ratio = self.sharpe(sum_vals)
 
+        self.more_stats(cum_df, sum_vals)
 
+        self.plotter(cum_dict, cum_df, sum_vals, sharpe_ratio)
